import gym
from dqn import Agent
import matplotlib.pyplot as plt
from collections import deque
import numpy as np
import torch
import torch.nn.functional as F
import torch.optim as optim

from bongsang.pid.env import BsEnv_v07 as ENV


# env = gym.make('CartPole-v1')
# env.seed(0)
# print('State shape: ', env.observation_space.shape)
# print('Number of actions: ', env.action_space.n)


Kp = 20
Ki = 4
sp = 25.5
pv_init = 26.5
loop = 90

env = ENV(mode='cooling', sp=sp, pv_init=pv_init, loop=loop, Kp_MIN=10, Kp_MAX=6000) 
env.make('EnergyBalance')   


agent = Agent(state_size=env.observation_size, action_size=env.action_size, seed=0)

# # watch an untrained agent
# state = env.reset()
# img = plt.imshow(env.render(mode='rgb_array'))
# for j in range(200):
#     action = agent.act(state)
#     img.set_data(env.render(mode='rgb_array')) 
#     plt.axis('off')
#     display.display(plt.gcf())
#     display.clear_output(wait=True)
#     state, reward, done, _ = env.step(action)
#     if done:
#         break 
        
# env.close()

def dqn(n_episodes=20000, max_t=1000, eps_start=1.0, eps_end=0.01, eps_decay=0.995):
    """Deep Q-Learning.
    
    Params
    ======
        n_episodes (int): maximum number of training episodes
        max_t (int): maximum number of timesteps per episode
        eps_start (float): starting value of epsilon, for epsilon-greedy action selection
        eps_end (float): minimum value of epsilon
        eps_decay (float): multiplicative factor (per episode) for decreasing epsilon
    """
    scores = []                        # list containing scores from each episode
    scores_window = deque(maxlen=100)  # last 100 scores
    eps = eps_start                    # initialize epsilon
    for i_episode in range(1, n_episodes+1):
        # state = env.reset()
        state, _, _ = env.step(0)
        score = 0
        done = False
        for t in range(max_t):
            action = agent.act(state, eps)
            next_state, reward, info = env.step(action)
            done = True
            agent.step(state, action, reward, next_state, done)
            state = next_state
            score += reward
            if done:
                break
        scores_window.append(score)       # save most recent score
        scores.append(score)              # save most recent score
        eps = max(eps_end, eps_decay*eps) # decrease epsilon
        print('\rEpisode {}\tAverage Score: {:.2f}'.format(i_episode, np.mean(scores_window)), end="")
        if i_episode % 100 == 0:
            print('\rEpisode {}\tAverage Score: {:.2f}'.format(i_episode, np.mean(scores_window)))
        # if np.mean(scores_window)>=200.0:
        #     print('\nEnvironment solved in {:d} episodes!\tAverage Score: {:.2f}'.format(i_episode-100, np.mean(scores_window)))
        #     torch.save(agent.qnetwork_local.state_dict(), 'checkpoint.pth')
        #     break
    return scores

scores = dqn()

# plot the scores
fig = plt.figure()
ax = fig.add_subplot(111)
plt.plot(np.arange(len(scores)), scores)
plt.ylabel('Score')
plt.xlabel('Episode #')
plt.show()
